@@ -28,6 +28,8 @@
 #include <stdlib.h>
 #include <colord/colord.h>
 
+#include "cd-cleanup.h"
+
 #define CD_PROFILE_DEFAULT_COPYRIGHT_STRING	"This profile is free of known copyright restrictions."
 
 typedef struct {
@@ -76,8 +78,8 @@ static void
 cd_util_add (GPtrArray *array, const gchar *name, const gchar *description, CdUtilPrivateCb callback)
 {
 	CdUtilItem *item;
-	gchar **names;
 	guint i;
+	_cleanup_strv_free_ gchar **names = NULL;
 
 	/* add each one */
 	names = g_strsplit (name, ",", -1);
@@ -94,7 +96,6 @@ cd_util_add (GPtrArray *array, const gchar *name, const gchar *description, CdUt
 		item->callback = callback;
 		g_ptr_array_add (array, item);
 	}
-	g_strfreev (names);
 }
 
 /**
@@ -149,17 +150,14 @@ static gboolean
 cd_util_run (CdUtilPrivate *priv, const gchar *command, gchar **values, GError **error)
 {
 	CdUtilItem *item;
-	gboolean ret = FALSE;
-	GString *string;
 	guint i;
+	_cleanup_string_free_ GString *string = NULL;
 
 	/* find command */
 	for (i = 0; i < priv->cmd_array->len; i++) {
 		item = g_ptr_array_index (priv->cmd_array, i);
-		if (g_strcmp0 (item->name, command) == 0) {
-			ret = item->callback (priv, values, error);
-			goto out;
-		}
+		if (g_strcmp0 (item->name, command) == 0)
+			return item->callback (priv, values, error);
 	}
 
 	/* not found */
@@ -172,9 +170,7 @@ cd_util_run (CdUtilPrivate *priv, const gchar *command, gchar **values, GError *
 		g_string_append_printf (string, " * %s\n", item->name);
 	}
 	g_set_error_literal (error, 1, 0, string->str);
-	g_string_free (string, TRUE);
-out:
-	return ret;
+	return TRUE;
 }
 
 /**
@@ -183,14 +179,10 @@ out:
 static gboolean
 cd_util_set_copyright (CdUtilPrivate *priv, gchar **values, GError **error)
 {
-	gboolean ret = TRUE;
-
-	/* check arguments */
 	if (g_strv_length (values) != 2) {
-		ret = FALSE;
 		g_set_error_literal (error, 1, 0,
 				     "invalid input, expect 'filename' 'value'");
-		goto out;
+		return FALSE;
 	}
 
 	/* set new value */
@@ -203,8 +195,7 @@ cd_util_set_copyright (CdUtilPrivate *priv, gchar **values, GError **error)
 				      priv->locale,
 				      values[1]);
 	}
-out:
-	return ret;
+	return TRUE;
 }
 
 /**
@@ -213,20 +204,16 @@ out:
 static gboolean
 cd_util_set_description (CdUtilPrivate *priv, gchar **values, GError **error)
 {
-	gboolean ret = TRUE;
-
 	/* check arguments */
 	if (g_strv_length (values) != 2) {
-		ret = FALSE;
 		g_set_error_literal (error, 1, 0,
 				     "invalid input, expect 'filename' 'value'");
-		goto out;
+		return FALSE;
 	}
 
 	/* set new value */
 	cd_icc_set_description (priv->icc, priv->locale, values[1]);
-out:
-	return ret;
+	return TRUE;
 }
 
 /**
@@ -235,20 +222,16 @@ out:
 static gboolean
 cd_util_set_manufacturer (CdUtilPrivate *priv, gchar **values, GError **error)
 {
-	gboolean ret = TRUE;
-
 	/* check arguments */
 	if (g_strv_length (values) != 2) {
-		ret = FALSE;
 		g_set_error_literal (error, 1, 0,
 				     "invalid input, expect 'filename' 'value'");
-		goto out;
+		return FALSE;
 	}
 
 	/* set new value */
 	cd_icc_set_manufacturer (priv->icc, priv->locale, values[1]);
-out:
-	return ret;
+	return TRUE;
 }
 
 /**
@@ -257,20 +240,16 @@ out:
 static gboolean
 cd_util_set_model (CdUtilPrivate *priv, gchar **values, GError **error)
 {
-	gboolean ret = TRUE;
-
 	/* check arguments */
 	if (g_strv_length (values) != 2) {
-		ret = FALSE;
 		g_set_error_literal (error, 1, 0,
 				     "invalid input, expect 'filename' 'value'");
-		goto out;
+		return FALSE;
 	}
 
 	/* set new value */
 	cd_icc_set_model (priv->icc, priv->locale, values[1]);
-out:
-	return ret;
+	return TRUE;
 }
 
 /**
@@ -279,13 +258,11 @@ out:
 static gboolean
 cd_util_clear_metadata (CdUtilPrivate *priv, gchar **values, GError **error)
 {
-	GHashTable *md;
+	_cleanup_hashtable_unref_ GHashTable *md = NULL;
 	md = cd_icc_get_metadata (priv->icc);
 	if (md == NULL)
-		goto out;
+		return TRUE;
 	g_hash_table_remove_all (md);
-	g_hash_table_unref (md);
-out:
 	return TRUE;
 }
 
@@ -297,29 +274,23 @@ cd_util_get_standard_space_filename (CdUtilPrivate *priv,
 				     CdStandardSpace standard_space,
 				     GError **error)
 {
-	CdProfile *profile_tmp = NULL;
-	gboolean ret;
 	gchar *filename = NULL;
+	_cleanup_object_unref_ CdProfile *profile_tmp = NULL;
 
 	/* try to find */
-	ret = cd_client_connect_sync (priv->client, NULL, error);
-	if (!ret)
-		goto out;
+	if (!cd_client_connect_sync (priv->client, NULL, error))
+		return NULL;
 	profile_tmp = cd_client_get_standard_space_sync (priv->client,
 							 standard_space,
 							 NULL,
 							 error);
 	if (profile_tmp == NULL)
-		goto out;
+		return NULL;
 
 	/* get filename */
-	ret = cd_profile_connect_sync (profile_tmp, NULL, error);
-	if (!ret)
-		goto out;
+	if (!cd_profile_connect_sync (profile_tmp, NULL, error))
+		return NULL;
 	filename = g_strdup (cd_profile_get_filename (profile_tmp));
-out:
-	if (profile_tmp != NULL)
-		g_object_unref (profile_tmp);
 	return filename;
 }
 
@@ -361,7 +332,7 @@ cd_util_get_coverage (cmsHPROFILE profile_proof,
 	/* get coverage */
 	ret = cd_icc_utils_get_coverage (icc, icc_ref, &coverage, error);
 	if (!ret)
-		goto out;
+		return FALSE;
 out:
 	if (file != NULL)
 		g_object_unref (file);
@@ -406,38 +377,33 @@ out:
 static gboolean
 cd_util_set_version (CdUtilPrivate *priv, gchar **values, GError **error)
 {
-	gboolean ret = TRUE;
 	gchar *endptr = NULL;
 	gdouble version;
 
 	/* check arguments */
 	if (g_strv_length (values) != 2) {
-		ret = FALSE;
 		g_set_error_literal (error, 1, 0,
 				     "invalid input, expect 'filename' 'version'");
-		goto out;
+		return FALSE;
 	}
 
 	/* get version */
 	version = g_ascii_strtod (values[1], &endptr);
 	if (endptr != NULL && endptr[0] != '\0') {
-		ret = FALSE;
 		g_set_error (error, 1, 0,
 			     "failed to parse version: '%s'",
 			     values[1]);
-		goto out;
+		return FALSE;
 	}
 	if (version < 1.0 || version > 6.0) {
-		ret = FALSE;
 		g_set_error (error, 1, 0,
 			     "invalid version %f", version);
-		goto out;
+		return FALSE;
 	}
 
 	/* set version */
 	cd_icc_set_version (priv->icc, version);
-out:
-	return ret;
+	return TRUE;
 }
 
 /**
@@ -446,24 +412,21 @@ out:
 static gboolean
 cd_util_export_tag_data (CdUtilPrivate *priv, gchar **values, GError **error)
 {
-	gboolean ret = TRUE;
-	GBytes *data = NULL;
-	gchar *out_fn = NULL;
+	gboolean ret;
+	_cleanup_bytes_unref_ GBytes *data = NULL;
+	_cleanup_free_ gchar *out_fn = NULL;
 
 	/* check arguments */
 	if (g_strv_length (values) != 2) {
-		ret = FALSE;
 		g_set_error_literal (error, 1, 0,
 				     "invalid input, expect 'filename' 'tag'");
-		goto out;
+		return FALSE;
 	}
 
 	/* get data */
 	data = cd_icc_get_tag_data (priv->icc, values[1], error);
-	if (data == NULL) {
-		ret = FALSE;
-		goto out;
-	}
+	if (data == NULL)
+		return FALSE;
 
 	/* save to file */
 	out_fn = g_strdup_printf ("./%s.bin", values[1]);
@@ -472,14 +435,10 @@ cd_util_export_tag_data (CdUtilPrivate *priv, gchar **values, GError **error)
 				   g_bytes_get_size (data),
 				   error);
 	if (!ret)
-		goto out;
+		return FALSE;
 	g_print ("Wrote %s\n", out_fn);
 	priv->rewrite_file = FALSE;
-out:
-	if (data != NULL)
-		g_bytes_unref (data);
-	g_free (out_fn);
-	return ret;
+	return TRUE;
 }
 
 /**
@@ -488,16 +447,14 @@ out:
 static gboolean
 cd_util_set_fix_metadata (CdUtilPrivate *priv, gchar **values, GError **error)
 {
-	gboolean ret = TRUE;
 	gchar *coverage_tmp;
 	gdouble coverage;
 
 	/* check arguments */
 	if (g_strv_length (values) != 1) {
-		ret = FALSE;
 		g_set_error_literal (error, 1, 0,
 				     "invalid input, expect 'filename'");
-		goto out;
+		return FALSE;
 	}
 
 	/* get coverages of common spaces */
@@ -507,10 +464,8 @@ cd_util_set_fix_metadata (CdUtilPrivate *priv, gchar **values, GError **error)
 		coverage = cd_util_get_profile_coverage (priv,
 							 CD_STANDARD_SPACE_ADOBE_RGB,
 							 error);
-		if (coverage < 0.0) {
-			ret = FALSE;
-			goto out;
-		}
+		if (coverage < 0.0)
+			return FALSE;
 		coverage_tmp = g_strdup_printf ("%f", coverage);
 		cd_icc_add_metadata (priv->icc,
 				     "GAMUT_coverage(adobe-rgb)",
@@ -522,10 +477,8 @@ cd_util_set_fix_metadata (CdUtilPrivate *priv, gchar **values, GError **error)
 		coverage = cd_util_get_profile_coverage (priv,
 							 CD_STANDARD_SPACE_SRGB,
 							 error);
-		if (coverage < 0.0) {
-			ret = FALSE;
-			goto out;
-		}
+		if (coverage < 0.0)
+			return FALSE;
 		coverage_tmp = g_strdup_printf ("%.2f", coverage);
 		cd_icc_add_metadata (priv->icc,
 				     "GAMUT_coverage(srgb)",
@@ -538,8 +491,7 @@ cd_util_set_fix_metadata (CdUtilPrivate *priv, gchar **values, GError **error)
 	cd_icc_add_metadata (priv->icc,
 			     CD_PROFILE_METADATA_CMF_VERSION,
 			     PACKAGE_VERSION);
-out:
-	return ret;
+	return TRUE;
 }
 
 /**
@@ -548,14 +500,11 @@ out:
 static gboolean
 cd_util_init_metadata (CdUtilPrivate *priv, gchar **values, GError **error)
 {
-	gboolean ret = TRUE;
-
 	/* check arguments */
 	if (g_strv_length (values) != 1) {
-		ret = FALSE;
 		g_set_error_literal (error, 1, 0,
 				     "invalid input, expect 'filename'");
-		goto out;
+		return FALSE;
 	}
 
 	/* add CMS defines */
@@ -568,8 +517,7 @@ cd_util_init_metadata (CdUtilPrivate *priv, gchar **values, GError **error)
 	cd_icc_add_metadata (priv->icc,
 			     CD_PROFILE_METADATA_CMF_VERSION,
 			     PACKAGE_VERSION);
-out:
-	return ret;
+	return TRUE;
 }
 
 /**
@@ -578,20 +526,16 @@ out:
 static gboolean
 cd_util_remove_metadata (CdUtilPrivate *priv, gchar **values, GError **error)
 {
-	gboolean ret = TRUE;
-
 	/* check arguments */
 	if (g_strv_length (values) != 2) {
-		ret = FALSE;
 		g_set_error_literal (error, 1, 0,
 				     "invalid input, expect 'filename' 'key'");
-		goto out;
+		return FALSE;
 	}
 
 	/* remove entry */
 	cd_icc_remove_metadata (priv->icc, values[1]);
-out:
-	return ret;
+	return TRUE;
 }
 
 /**
@@ -600,20 +544,16 @@ out:
 static gboolean
 cd_util_add_metadata (CdUtilPrivate *priv, gchar **values, GError **error)
 {
-	gboolean ret = TRUE;
-
 	/* check arguments */
 	if (g_strv_length (values) != 3) {
-		ret = FALSE;
 		g_set_error_literal (error, 1, 0,
 				     "invalid input, expect 'filename' 'key' 'value'");
-		goto out;
+		return FALSE;
 	}
 
 	/* add new entry */
 	cd_icc_add_metadata (priv->icc, values[1], values[2]);
-out:
-	return ret;
+	return TRUE;
 }
 
 /**
@@ -625,35 +565,31 @@ cd_util_extract_vcgt (CdUtilPrivate *priv, gchar **values, GError **error)
 	cmsFloat32Number in;
 	cmsHPROFILE lcms_profile;
 	const cmsToneCurve **vcgt;
-	gboolean ret = TRUE;
 	guint i;
 	guint size;
 
 	/* check arguments */
 	if (g_strv_length (values) != 2) {
-		ret = FALSE;
 		g_set_error_literal (error, 1, 0,
 				     "invalid input, expect 'filename' size'");
-		goto out;
+		return FALSE;
 	}
 
 	/* invalid size */
 	size = atoi (values[1]);
 	if (size <= 1 || size > 1024) {
-		ret = FALSE;
 		g_set_error_literal (error, 1, 0,
 				     "invalid size,expected 2-1024");
-		goto out;
+		return FALSE;
 	}
 
 	/* does profile have VCGT */
 	lcms_profile = cd_icc_get_handle (priv->icc);
 	vcgt = cmsReadTag (lcms_profile, cmsSigVcgtTag);
 	if (vcgt == NULL || vcgt[0] == NULL) {
-		ret = FALSE;
 		g_set_error_literal (error, 1, 0,
 				     "profile does not have any VCGT data");
-		goto out;
+		return FALSE;
 	}
 
 	/* output data */
@@ -668,9 +604,7 @@ cd_util_extract_vcgt (CdUtilPrivate *priv, gchar **values, GError **error)
 
 	/* success */
 	priv->rewrite_file = FALSE;
-	ret = TRUE;
-out:
-	return ret;
+	return TRUE;
 }
 
 /**
@@ -702,11 +636,11 @@ main (int argc, char *argv[])
 	CdUtilPrivate *priv;
 	gboolean ret = TRUE;
 	gboolean verbose = FALSE;
-	gchar *cmd_descriptions = NULL;
-	gchar *locale = NULL;
-	GError *error = NULL;
-	GFile *file = NULL;
 	guint retval = 1;
+	_cleanup_error_free_ GError *error = NULL;
+	_cleanup_free_ gchar *cmd_descriptions = NULL;
+	_cleanup_free_ gchar *locale = NULL;
+	_cleanup_object_unref_ GFile *file = NULL;
 	const GOptionEntry options[] = {
 		{ "verbose", 'v', 0, G_OPTION_ARG_NONE, &verbose,
 			/* TRANSLATORS: command line option */
@@ -807,10 +741,8 @@ main (int argc, char *argv[])
 	ret = g_option_context_parse (priv->context, &argc, &argv, &error);
 	if (!ret) {
 		/* TRANSLATORS: the user didn't read the man page */
-		g_print ("%s: %s\n",
-			 _("Failed to parse arguments"),
+		g_print ("%s: %s\n", _("Failed to parse arguments"),
 			 error->message);
-		g_error_free (error);
 		goto out;
 	}
 
@@ -841,7 +773,6 @@ main (int argc, char *argv[])
 				&error);
 	if (!ret) {
 		g_print ("%s\n", error->message);
-		g_error_free (error);
 		goto out;
 	}
 
@@ -849,7 +780,6 @@ main (int argc, char *argv[])
 	ret = cd_util_run (priv, argv[2], (gchar**) &argv[2], &error);
 	if (!ret) {
 		g_print ("%s\n", error->message);
-		g_error_free (error);
 		goto out;
 	}
 
@@ -862,7 +792,6 @@ main (int argc, char *argv[])
 					&error);
 		if (!ret) {
 			g_print ("%s\n", error->message);
-			g_error_free (error);
 			goto out;
 		}
 	}
@@ -880,10 +809,6 @@ out:
 		g_free (priv->locale);
 		g_free (priv);
 	}
-	if (file != NULL)
-		g_object_unref (file);
-	g_free (locale);
-	g_free (cmd_descriptions);
 	return retval;
 }
 
