@@ -28,6 +28,8 @@
 
 #include <glib.h>
 
+G_BEGIN_DECLS
+
 typedef enum {
 	CD_BUFFER_KIND_REQUEST,
 	CD_BUFFER_KIND_RESPONSE,
@@ -50,5 +52,7 @@ void		 cd_buffer_write_uint32_be	(guint8		*buffer,
 void		 cd_buffer_write_uint32_le	(guint8		*buffer,
 						 guint32	 value);
 
+G_END_DECLS
+
 #endif /* __CD_BUFFER_H__ */
 
