@@ -70,7 +70,9 @@ static const CdEnumMatch enum_sensor_kind[] = {
 	{CD_SENSOR_KIND_SPYDER2,			"spyder2"},
 	{CD_SENSOR_KIND_SPYDER3,			"spyder3"},
 	{CD_SENSOR_KIND_SPYDER4,			"spyder4"},
+	{CD_SENSOR_KIND_SPYDER5,			"spyder5"},
 	{CD_SENSOR_KIND_SPYDER,				"spyder"},
+	{CD_SENSOR_KIND_SPARK,				"spark"},
 	{0, NULL}
 };
 
