@@ -66,8 +66,8 @@ struct _CdIccStoreClass
  * Since: 1.1.1
  **/
 typedef enum {
-	CD_ICC_STORE_SEARCH_FLAGS_NONE			= 0,
-	CD_ICC_STORE_SEARCH_FLAGS_CREATE_LOCATION	= 1,
+	CD_ICC_STORE_SEARCH_FLAGS_NONE			= 0,	/* Since: 1.0.2 */
+	CD_ICC_STORE_SEARCH_FLAGS_CREATE_LOCATION	= 1,	/* Since: 1.0.2 */
 	/*< private >*/
 	CD_ICC_STORE_SEARCH_FLAGS_LAST
 } CdIccStoreSearchFlags;
@@ -83,9 +83,9 @@ typedef enum {
  * Since: 1.1.1
  **/
 typedef enum {
-	CD_ICC_STORE_SEARCH_KIND_SYSTEM,
-	CD_ICC_STORE_SEARCH_KIND_MACHINE,
-	CD_ICC_STORE_SEARCH_KIND_USER,
+	CD_ICC_STORE_SEARCH_KIND_SYSTEM,		/* Since: 1.0.2 */
+	CD_ICC_STORE_SEARCH_KIND_MACHINE,		/* Since: 1.0.2 */
+	CD_ICC_STORE_SEARCH_KIND_USER,			/* Since: 1.0.2 */
 	/*< private >*/
 	CD_ICC_STORE_SEARCH_KIND_LAST
 } CdIccStoreSearchKind;
