@@ -461,8 +461,14 @@ ch_client_get_default (GError **error)
 	}
 	device = g_usb_context_find_by_vid_pid (usb_ctx,
 						CH_USB_VID,
-						CH_USB_PID_FIRMWARE,
-						error);
+						CH_USB_PID_FIRMWARE_ALS_SENSOR_HID,
+						NULL);
+	if (device == NULL) {
+		device = g_usb_context_find_by_vid_pid (usb_ctx,
+							CH_USB_VID,
+							CH_USB_PID_FIRMWARE,
+							error);
+	}
 	if (device == NULL)
 		goto out;
 	g_debug ("Found ColorHug device %s",
@@ -506,7 +512,7 @@ ch_test_state_func (void)
 	device_queue = ch_device_queue_new ();
 	ch_device_queue_set_leds (device_queue,
 				  device,
-				  3,
+				  1,
 				  0,
 				  0x00,
 				  0x00);
@@ -525,7 +531,7 @@ ch_test_state_func (void)
 				       &error);
 	g_assert_no_error (error);
 	g_assert (ret);
-	g_assert_cmpint (leds, ==, 3);
+	g_assert_cmpint (leds, ==, 1);
 
 	/* verify color select */
 	if (ch_device_get_mode (device) == CH_DEVICE_MODE_FIRMWARE) {
@@ -636,6 +642,11 @@ ch_test_state_func (void)
 		g_timer_destroy (timer);
 	}
 
+	/* close */
+	ret = ch_device_close (device, &error);
+	g_assert_no_error (error);
+	g_assert (ret);
+
 	g_object_unref (device_queue);
 }
 
@@ -662,6 +673,13 @@ ch_test_eeprom_func (void)
 
 	/* load the device */
 	device = ch_client_get_default (&error);
+	if (device != NULL && g_usb_device_get_pid (device) != CH_USB_PID_FIRMWARE) {
+		ret = ch_device_close (device, &error);
+		g_assert_no_error (error);
+		g_assert (ret);
+		g_debug ("not capable device, skipping tests");
+		return;
+	}
 	if (device == NULL && g_error_matches (error,
 					       G_USB_DEVICE_ERROR,
 					       G_USB_DEVICE_ERROR_NO_DEVICE)) {
@@ -899,6 +917,9 @@ ch_test_eeprom_func (void)
 	g_assert (ret);
 #endif
 
+	ret = ch_device_close (device, &error);
+	g_assert_no_error (error);
+	g_assert (ret);
 	g_object_unref (device);
 	g_object_unref (device_queue);
 }
@@ -964,6 +985,11 @@ ch_test_reading_func (void)
 	g_assert (ret);
 	g_assert_cmpint (take_reading, >, 0);
 
+	/* close */
+	ret = ch_device_close (device, &error);
+	g_assert_no_error (error);
+	g_assert (ret);
+
 	g_object_unref (device);
 	g_object_unref (device_queue);
 }
@@ -986,6 +1012,13 @@ ch_test_reading_xyz_func (void)
 
 	/* load the device */
 	device = ch_client_get_default (&error);
+	if (device != NULL && g_usb_device_get_pid (device) != CH_USB_PID_FIRMWARE) {
+		ret = ch_device_close (device, &error);
+		g_assert_no_error (error);
+		g_assert (ret);
+		g_debug ("not capable device, skipping tests");
+		return;
+	}
 	if (device == NULL && g_error_matches (error,
 					       G_USB_DEVICE_ERROR,
 					       G_USB_DEVICE_ERROR_NO_DEVICE)) {
@@ -1098,6 +1131,12 @@ ch_test_reading_xyz_func (void)
 		g_assert_cmpfloat (scaling_factor_actual, <, 1.1);
 		reading1.Z = reading2.Z * 2;
 	}
+
+	/* close */
+	ret = ch_device_close (device, &error);
+	g_assert_no_error (error);
+	g_assert (ret);
+
 	g_object_unref (device);
 	g_object_unref (device_queue);
 }
@@ -1121,6 +1160,13 @@ ch_test_incomplete_request_func (void)
 
 	/* load the device */
 	device = ch_client_get_default (&error);
+	if (device != NULL && g_usb_device_get_pid (device) != CH_USB_PID_FIRMWARE) {
+		ret = ch_device_close (device, &error);
+		g_assert_no_error (error);
+		g_assert (ret);
+		g_debug ("not capable device, skipping tests");
+		return;
+	}
 	if (device == NULL && g_error_matches (error,
 					       G_USB_DEVICE_ERROR,
 					       G_USB_DEVICE_ERROR_NO_DEVICE)) {
@@ -1186,8 +1232,12 @@ ch_test_incomplete_request_func (void)
 	g_assert_cmpint (buffer[0], ==, CH_ERROR_INCOMPLETE_REQUEST);
 	g_assert_cmpint (buffer[1], ==, CH_CMD_GET_FIRMWARE_VERSION);
 out:
-	if (device != NULL)
+	if (device != NULL) {
+		ret = ch_device_close (device, &error);
+		g_assert_no_error (error);
+		g_assert (ret);
 		g_object_unref (device);
+	}
 }
 
 /**
