@@ -24,7 +24,6 @@
 #include <gio/gio.h>
 #include <glib-object.h>
 #include <sqlite3.h>
-#include <syslog.h>
 
 #include "cd-cleanup.h"
 #include "cd-common.h"
@@ -65,7 +64,7 @@ cd_profile_db_load (CdProfileDb *pdb,
 		return FALSE;
 
 	g_debug ("CdProfileDb: trying to open database '%s'", filename);
-	syslog (LOG_INFO, "Using profile database file %s", filename);
+	g_info ("Using profile database file %s", filename);
 	rc = sqlite3_open (filename, &pdb->priv->db);
 	if (rc != SQLITE_OK) {
 		g_set_error (error,
