@@ -28,7 +28,7 @@
 #include <gio/gio.h>
 #include <colord-private.h>
 
-#ifdef HAVE_GUDEV
+#ifdef HAVE_UDEV
 #include <gudev/gudev.h>
 #endif
 
@@ -84,14 +84,14 @@ gboolean	 cd_sensor_register_object	(CdSensor		*sensor,
 						 GDBusConnection	*connection,
 						 GDBusInterfaceInfo	*info,
 						 GError			**error);
-#ifdef HAVE_GUDEV
+#ifdef HAVE_UDEV
 gboolean	 cd_sensor_set_from_device	(CdSensor		*sensor,
 						 GUdevDevice		*device,
 						 GError			**error);
 GUdevDevice	*cd_sensor_get_device		(CdSensor		*sensor);
+#endif
 void		 cd_sensor_set_index		(CdSensor		*sensor,
 						 guint			 idx);
-#endif
 #ifdef HAVE_GUSB
 GUsbDevice	*cd_sensor_open_usb_device	(CdSensor		*sensor,
 						 gint			 config,
